package com.bs.df.controller;

import java.util.*;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;

import com.bs.common.core.domain.entity.SysUser;
import com.bs.df.domain.DfBroker;
import com.bs.df.domain.DfBrokerProduct;
import com.bs.df.domain.DfProductPoster;
import com.bs.df.service.IDfBrokerProductService;
import com.bs.df.service.IDfBrokerService;
import com.bs.df.service.IDfProductPosterService;
import com.bs.df.utils.HtmlUtils;
import com.bs.system.domain.DfArea;
import com.bs.system.service.IDfAreaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.bs.common.annotation.Log;
import com.bs.common.core.controller.BaseController;
import com.bs.common.core.domain.AjaxResult;
import com.bs.common.core.page.TableDataInfo;
import com.bs.common.enums.BusinessType;
import com.bs.common.utils.poi.ExcelUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import cn.hutool.core.lang.Validator;
import com.bs.df.domain.DfProductInfo;
import com.bs.df.service.IDfProductInfoService;
import javax.annotation.Resource;

/**
 * 产品信息Controller
 *
 * @author bs
 * @date 2024-04-06
 */
@Api(tags = "产品信息")
@RestController
@RequestMapping("/product/info")
public class DfProductInfoController extends BaseController {
    @Resource
    private IDfProductInfoService dfProductInfoService;
    @Resource
    private IDfProductPosterService dfProductPosterService;
    @Resource
    private IDfAreaService dfAreaService;
    @Resource
    private IDfBrokerProductService dfBrokerProductService;
    @Resource
    private IDfBrokerService dfBrokerService;



    /**
     * 分页查询产品信息列表
     */
    @ApiOperation("分页查询产品信息列表")
    @GetMapping("/pageList")
    public TableDataInfo pageList(DfProductInfo dfProductInfo) {
        startPage();
        LambdaQueryWrapper<DfProductInfo> queryWrapper = new LambdaQueryWrapper();
        String brokerIds = dfProductInfo.getBrokerIds();
        if ("1".equals(dfProductInfo.getSelectByBroker())) {
            queryWrapper.inSql(DfProductInfo::getId,"select product_id from df_broker_product where broker_id in (" + brokerIds + ") and del_flag = '0'");
        }
        condition(queryWrapper,dfProductInfo);
        List<DfProductInfo> list = dfProductInfoService.list(queryWrapper);
        Map<String, String> cityMap = new HashMap<>();
        List<DfArea> allCities = dfAreaService.list();
        for (DfArea city : allCities) {
            cityMap.put(city.getAreaId(), city.getName());
        }
        Map<Long, String> brokerProductMap = new HashMap<>();
        if ("1".equals(dfProductInfo.getSelectByBroker())) {
            List<DfBroker> listBroker = dfBrokerService.list();
            Map<Long, String> brokerMap = new HashMap<>();
            for (DfBroker broker : listBroker) {
                brokerMap.put(broker.getId(), broker.getBrokerName());
            }
            List<DfBrokerProduct> listBrokerProduct = dfBrokerProductService.list();
            for (DfBrokerProduct brokerProduct : listBrokerProduct) {
                String brokerName = brokerMap.get(brokerProduct.getBrokerId());
                brokerProduct.setBrokerName(brokerName);
            }
            for (DfBrokerProduct brokerProduct : listBrokerProduct) {
                brokerProductMap.put(brokerProduct.getProductId(), brokerProduct.getBrokerName());
            }
        }
        for (DfProductInfo productInfo : list) {
            productInfo.setInterestRate(productInfo.getInterestRateBegin() + "%-" + productInfo.getInterestRateEnd() + "%");
            productInfo.setProductIntro(HtmlUtils.stripHtmlTags(productInfo.getProductIntro()));
            productInfo.setProductDetail(HtmlUtils.stripHtmlTags(productInfo.getProductDetail()));
            String businessCity = productInfo.getBusinessCity(); // 获取原始的城市ID字符串
            if (Validator.isNotEmpty(businessCity)) {
                String[] cityIds = businessCity.split(",");
                StringBuilder translatedCities = new StringBuilder();
                for (String cityId : cityIds) {
                    String cityName = cityMap.get(cityId); // 从 Map 中获取城市名称
                    if (cityName != null) {
                        translatedCities.append(cityName).append(",");
                    }
                }
                if (translatedCities.length() > 0) {
                    translatedCities.deleteCharAt(translatedCities.length() - 1);
                }
                productInfo.setBusinessCity(translatedCities.toString());
            }
            if ("1".equals(dfProductInfo.getSelectByBroker())) {
                productInfo.setBrokerName(brokerProductMap.get(productInfo.getId()));
            }
        }
        return getDataTable(list);
    }

    /**
     * 查询产品信息列表
     */
    @ApiOperation("查询产品信息列表")
    @GetMapping("/list")
    public AjaxResult list(DfProductInfo dfProductInfo) {
        LambdaQueryWrapper<DfProductInfo> queryWrapper = new LambdaQueryWrapper();
        if ("1".equals(dfProductInfo.getSelectByBroker())) {
            queryWrapper.notInSql(DfProductInfo::getId,"select product_id from df_broker_product where del_flag = '0'")
                    .or()
                    .eq(DfProductInfo::getId,dfProductInfo.getId());
        }
        condition(queryWrapper,dfProductInfo);
        List<DfProductInfo> list = dfProductInfoService.list(queryWrapper);
        return success(list);
    }

    /**
     * 导出产品信息列表
     */
    @ApiOperation("导出产品信息列表")
    @Log(title = "产品信息导出", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DfProductInfo dfProductInfo) {
        Map<String, String> cityMap = new HashMap<>();
        List<DfArea> allCities = dfAreaService.list();
        for (DfArea city : allCities) {
            cityMap.put(city.getAreaId(), city.getName());
        }
        Map<Long, String> brokerProductMap = new HashMap<>();
        List<DfBroker> listBroker = dfBrokerService.list();
        Map<Long, String> brokerMap = new HashMap<>();
        for (DfBroker broker : listBroker) {
            brokerMap.put(broker.getId(), broker.getBrokerName());
        }
        List<DfBrokerProduct> listBrokerProduct = dfBrokerProductService.list();
        for (DfBrokerProduct brokerProduct : listBrokerProduct) {
            String brokerName = brokerMap.get(brokerProduct.getBrokerId());
            brokerProduct.setBrokerName(brokerName);
        }
        for (DfBrokerProduct brokerProduct : listBrokerProduct) {
            brokerProductMap.put(brokerProduct.getProductId(), brokerProduct.getBrokerName());
        }
        LambdaQueryWrapper<DfProductInfo> queryWrapper = new LambdaQueryWrapper();
        condition(queryWrapper,dfProductInfo);
        List<DfProductInfo> list = dfProductInfoService.list(queryWrapper);
        for (DfProductInfo productInfo : list) {
            productInfo.setInterestRate(productInfo.getInterestRateBegin() + "%-" + productInfo.getInterestRateEnd() + "%");
            productInfo.setProductIntro(HtmlUtils.stripHtmlTags(productInfo.getProductIntro()));
            productInfo.setProductDetail(HtmlUtils.stripHtmlTags(productInfo.getProductDetail()));
            String businessCity = productInfo.getBusinessCity(); // 获取原始的城市ID字符串
            if (Validator.isNotEmpty(businessCity)) {
                String[] cityIds = businessCity.split(","); // 按逗号分割城市ID
                StringBuilder translatedCities = new StringBuilder();
                for (String cityId : cityIds) {
                    String cityName = cityMap.get(cityId); // 从 Map 中获取城市名称
                    if (cityName != null) {
                        translatedCities.append(cityName).append(",");
                    }
                }
                if (translatedCities.length() > 0) {
                    translatedCities.deleteCharAt(translatedCities.length() - 1);
                }
                productInfo.setBusinessCity(translatedCities.toString());
                productInfo.setBrokerName(brokerProductMap.get(productInfo.getId()));
            }
        }
        ExcelUtil<DfProductInfo> util = new ExcelUtil<DfProductInfo>(DfProductInfo. class);
        util.exportExcel(response, list, "产品信息数据");
    }

    /**
     * 获取产品信息详细信息
     */
    @ApiOperation("获取产品信息详细信息")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        DfProductInfo byId = dfProductInfoService.getById(id);
        List<DfBrokerProduct> list = dfBrokerProductService.list(new LambdaQueryWrapper<DfBrokerProduct>().eq(DfBrokerProduct::getProductId, id));
        if (list.size() > 0 && list != null) {
            byId.setBrokerId(list.get(0).getBrokerId());
        }
        String businessCity = byId.getBusinessCity();
        if (Validator.isEmpty(businessCity)) {
            return success(byId);
        }
        List<String> businessCityList = Arrays.asList(businessCity.split(","));
        List<DfArea> douAreas = dfAreaService.listByIds(businessCityList);
        String concatenatedNames = douAreas.stream()
                .map(DfArea::getName)
                .collect(Collectors.joining(","));
        byId.setBusinessCityByName(concatenatedNames);
        return success(byId);
    }

    /**
     * 新增产品信息
     */
    @ApiOperation("新增产品信息")
    @Log(title = "产品信息新增", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody DfProductInfo dfProductInfo) {
        return toAjax(dfProductInfoService.save(dfProductInfo));
    }

    /**
     * 修改产品信息
     */
    @ApiOperation("修改产品信息")
    @Log(title = "产品信息修改", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody DfProductInfo dfProductInfo) {
        return toAjax(dfProductInfoService.updateById(dfProductInfo));
    }



    /**
     * 删除产品信息
     */
    @ApiOperation("删除产品信息")
    @Log(title = "产品信息删除", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        List<DfProductPoster> list = dfProductPosterService.list(new LambdaQueryWrapper<DfProductPoster>().in(DfProductPoster::getProductId, ids));
        if(list.size() > 0){
            return AjaxResult.warn("该产品下有海报，请先删除海报");
        }
        return toAjax(dfProductInfoService.removeBatchByIds(ids));
    }

    /**
     * 修改产品状态
     */
    @ApiOperation("修改产品状态")
    @Log(title = "修改产品状态", businessType = BusinessType.DELETE)
    @PutMapping("/changeStatus")
    public AjaxResult changeStatus(@RequestBody DfProductInfo dfProductInfo) {
        Long[] ids = dfProductInfo.getIds();
        String status = dfProductInfo.getStatus();
        List<DfProductInfo> dfProductInfos = new ArrayList<>();
        for (Long id : ids) {
            DfProductInfo dfProductInfoNew = new DfProductInfo();
            dfProductInfoNew.setId(id);
            dfProductInfoNew.setStatus(status);
            dfProductInfos.add(dfProductInfoNew);
        }
        return toAjax(dfProductInfoService.updateBatchById(dfProductInfos));
    }




    /**
     * 修改代理产品权限
     */
    @ApiOperation("修改代理产品权限")
    @Log(title = "代理产品权限修改", businessType = BusinessType.UPDATE)
    @PutMapping("/editByBroker")
    public AjaxResult editByBroker(@RequestBody DfProductInfo dfProductInfo) {
        Long productInfoId = dfProductInfo.getId();
        Long brokerId = dfProductInfo.getBrokerId();
        List<DfBrokerProduct> list = dfBrokerProductService.list(new LambdaQueryWrapper<DfBrokerProduct>().eq(DfBrokerProduct::getProductId, productInfoId));
        if (list != null && list.size() > 0) {
            DfBrokerProduct dfBrokerProduct = new DfBrokerProduct().setBrokerId(brokerId).setProductId(productInfoId).setId(list.get(0).getId());
            return toAjax(dfBrokerProductService.updateById(dfBrokerProduct));
        } else {
            DfBrokerProduct dfBrokerProduct = new DfBrokerProduct().setBrokerId(brokerId).setProductId(productInfoId);
            return toAjax(dfBrokerProductService.save(dfBrokerProduct));
        }
    }

    /**
     * 移除代理产品权限
     */
    @ApiOperation("移除代理产品权限")
    @Log(title = "代理产品权限移除", businessType = BusinessType.DELETE)
    @DeleteMapping("/removeByBroker/{ids}")
    public AjaxResult removeByBroker(@PathVariable List<Long> ids) {
        return toAjax(dfBrokerProductService.remove(new LambdaQueryWrapper<DfBrokerProduct>().in(DfBrokerProduct::getProductId, ids)));
    }

    /**
      * 条件设置
      */
    private void condition (LambdaQueryWrapper<DfProductInfo> queryWrapper,DfProductInfo dfProductInfo){


        //产品id
        if(Validator.isNotEmpty(dfProductInfo.getId())){
            queryWrapper.eq(DfProductInfo::getId,dfProductInfo.getId());
        }

        //产品名称
        if(Validator.isNotEmpty(dfProductInfo.getProductName())){
            queryWrapper.like(DfProductInfo::getProductName,dfProductInfo.getProductName());
        }

        //产品简称
        if(Validator.isNotEmpty(dfProductInfo.getSimpleName())){
            queryWrapper.like(DfProductInfo::getSimpleName,dfProductInfo.getSimpleName());
        }

        //产品类型
        if(Validator.isNotEmpty(dfProductInfo.getProductType())){
            queryWrapper.eq(DfProductInfo::getProductType,dfProductInfo.getProductType());
        }

        //产品细类
        if(Validator.isNotEmpty(dfProductInfo.getProductSubtype())){
            queryWrapper.eq(DfProductInfo::getProductSubtype,dfProductInfo.getProductSubtype());
        }

        //最高额度（元）
        if(Validator.isNotEmpty(dfProductInfo.getMaxAmount())){
            queryWrapper.eq(DfProductInfo::getMaxAmount,dfProductInfo.getMaxAmount());
        }

        if(Validator.isNotEmpty(dfProductInfo.getMaxAmountBegin())){
            queryWrapper.ge(DfProductInfo::getMaxAmount,dfProductInfo.getMaxAmountBegin());
        }

        if(Validator.isNotEmpty(dfProductInfo.getMaxAmountEnd())){
            queryWrapper.le(DfProductInfo::getMaxAmount,dfProductInfo.getMaxAmountEnd());
        }

        //年利率（单利）
        if(Validator.isNotEmpty(dfProductInfo.getInterestRateBegin())){
            queryWrapper.ge(DfProductInfo::getInterestRateBegin,dfProductInfo.getInterestRateBegin());
        }

        if(Validator.isNotEmpty(dfProductInfo.getInterestRateEnd())){
            queryWrapper.le(DfProductInfo::getInterestRateEnd,dfProductInfo.getInterestRateEnd());
        }

        //产品简介
        if(Validator.isNotEmpty(dfProductInfo.getProductIntro())){
            queryWrapper.eq(DfProductInfo::getProductIntro,dfProductInfo.getProductIntro());
        }

        //产品详情（富文本）
        if(Validator.isNotEmpty(dfProductInfo.getProductDetail())){
            queryWrapper.eq(DfProductInfo::getProductDetail,dfProductInfo.getProductDetail());
        }

        //上架时间起
        if(Validator.isNotEmpty(dfProductInfo.getListBegin())){
            queryWrapper.ge(DfProductInfo::getListBegin,dfProductInfo.getListBegin());
        }

        //上架时间止
        if(Validator.isNotEmpty(dfProductInfo.getListEnd())){
            queryWrapper.le(DfProductInfo::getListEnd,dfProductInfo.getListEnd());
        }

        if(Validator.isNotEmpty(dfProductInfo.getParams().get("beginTime"))){
            queryWrapper.ge(DfProductInfo::getListBegin,dfProductInfo.getParams().get("beginTime"));
        }

        if(Validator.isNotEmpty(dfProductInfo.getParams().get("endTime"))){
            queryWrapper.le(DfProductInfo::getListEnd,dfProductInfo.getParams().get("endTime"));
        }

        //还款周期
        if(Validator.isNotEmpty(dfProductInfo.getRepaymentCycle())){
            queryWrapper.eq(DfProductInfo::getRepaymentCycle,dfProductInfo.getRepaymentCycle());
        }

        //还款方式
        if(Validator.isNotEmpty(dfProductInfo.getRepaymentMethod())){
            queryWrapper.eq(DfProductInfo::getRepaymentMethod,dfProductInfo.getRepaymentMethod());
        }

        //展业城市
        if(Validator.isNotEmpty(dfProductInfo.getBusinessCity())){
            queryWrapper.eq(DfProductInfo::getBusinessCity,dfProductInfo.getBusinessCity());
        }

        //综合费率
        if(Validator.isNotEmpty(dfProductInfo.getCombinedRates())){
            queryWrapper.eq(DfProductInfo::getCombinedRates,dfProductInfo.getCombinedRates());
        }

        //状态0草稿1上架
        if(Validator.isNotEmpty(dfProductInfo.getStatus())){
            queryWrapper.eq(DfProductInfo::getStatus,dfProductInfo.getStatus());
        }

        //备注
        if(Validator.isNotEmpty(dfProductInfo.getRemark())){
            queryWrapper.eq(DfProductInfo::getRemark,dfProductInfo.getRemark());
        }

        //创建部门
        if(Validator.isNotEmpty(dfProductInfo.getCreateDept())){
            queryWrapper.eq(DfProductInfo::getCreateDept,dfProductInfo.getCreateDept());
        }

        //创建人员
        if(Validator.isNotEmpty(dfProductInfo.getCreateBy())){
            queryWrapper.eq(DfProductInfo::getCreateBy,dfProductInfo.getCreateBy());
        }

        //创建时间
        if(Validator.isNotEmpty(dfProductInfo.getCreateTime())){
            queryWrapper.eq(DfProductInfo::getCreateTime,dfProductInfo.getCreateTime());
        }

        //修改人员
        if(Validator.isNotEmpty(dfProductInfo.getUpdateBy())){
            queryWrapper.eq(DfProductInfo::getUpdateBy,dfProductInfo.getUpdateBy());
        }

        //修改时间
        if(Validator.isNotEmpty(dfProductInfo.getUpdateTime())){
            queryWrapper.eq(DfProductInfo::getUpdateTime,dfProductInfo.getUpdateTime());
        }

        //删除标志（0代表存在 2代表删除）
        if(Validator.isNotEmpty(dfProductInfo.getDelFlag())){
            queryWrapper.eq(DfProductInfo::getDelFlag,dfProductInfo.getDelFlag());
        }

        //租户id
        if(Validator.isNotEmpty(dfProductInfo.getTenantId())){
            queryWrapper.eq(DfProductInfo::getTenantId,dfProductInfo.getTenantId());
        }

        }

}
